@@ -5,8 +5,6 @@
 # means to set Unix file permissions and ownerships on the resulting
 # file. Run perldoc(1) on this module for more information.
 #
-# Copyright 2009-2010,2012-2013 by Jeremy Mates.
-#
 # This module is free software; you can redistribute it and/or modify it
 # under the Artistic license.
 
@@ -21,7 +19,7 @@ use File::Path qw/mkpath/;
 use File::Temp qw/tempfile/;
 use IO::Handle;
 
-our $VERSION = '1.17';
+our $VERSION = '1.18';
 
 # Default options
 my %default_params = ( MKPATH => 0, template => ".tmp.XXXXXXXXXX" );
@@ -269,11 +267,14 @@ sub _resolve {
   undef $tmp_fh;
 
   if ( exists $params_ref->{mode} ) {
+    my $mode = $params_ref->{mode};
     croak 'invalid mode data'
-      if !defined $params_ref->{mode}
-        or $params_ref->{mode} !~ m/^[0-9]+$/;
+      if !defined $mode
+      or $mode !~ m/^[0-9]+$/;
 
-    my $count = chmod( $params_ref->{mode}, $tmp_filename );
+    my $int_mode = substr( $mode, 0, 1 ) eq '0' ? oct($mode) : ( $mode + 0 );
+
+    my $count = chmod( $int_mode, $tmp_filename );
     if ( $count != 1 ) {
       my $save_errstr = $!;
       _cleanup( $tmp_fh, $tmp_filename );
@@ -291,10 +292,10 @@ sub _resolve {
 
   if ( exists $params_ref->{mtime} ) {
     croak 'invalid mtime data'
-      if ! defined $params_ref->{mtime}
-        or $params_ref->{mtime} !~ m/^[0-9]+$/;
+      if !defined $params_ref->{mtime}
+      or $params_ref->{mtime} !~ m/^[0-9]+$/;
 
-    my ( $file_atime ) = ( stat $tmp_filename )[ 8 ];
+    my ($file_atime) = ( stat $tmp_filename )[8];
     my $count = utime( $file_atime, $params_ref->{mtime}, $tmp_filename );
     if ( $count != 1 ) {
       my $save_errstr = $!;
@@ -303,6 +304,32 @@ sub _resolve {
     }
   }
 
+  # If the file does not exist, but the backup does;
+  # the backup is left unmodified
+  if ( exists $params_ref->{backup} && -f $params_ref->{file} ) {
+    croak 'invalid backup suffix'
+      if !defined $params_ref->{backup}
+      or $params_ref->{backup} eq '';
+
+    # The backup file will be hardlinked in same directory as original
+    my $backup_filename = $params_ref->{file} . $params_ref->{backup};
+    if ( -f $backup_filename ) {
+      my $count = unlink($backup_filename);
+      if ( $count != 1 ) {
+        my $save_errstr = $!;
+        _cleanup( $tmp_fh, $tmp_filename );
+        die "unable to unlink existing backup file: $save_errstr\n";
+      }
+    }
+
+    # Make hardlink
+    if ( !link( $params_ref->{file}, $backup_filename ) ) {
+      my $save_errstr = $!;
+      _cleanup( $tmp_fh, $tmp_filename );
+      die "unable to link existing file to backup file: $save_errstr\n";
+    }
+  }
+
   unless ( rename( $tmp_filename, $params_ref->{file} ) ) {
     my $save_errstr = $!;
     _cleanup( $tmp_fh, $tmp_filename );
@@ -384,18 +411,22 @@ sub _set_ownership {
   my ( $user_name, $group_name ) = split /[:.]/, $owner, 2;
 
   my ( $login, $pass, $user_uid, $user_gid );
-  if ( $user_name =~ m/^([0-9]+)$/ ) {
-    $uid = $1;
-  } else {
-    ( $login, $pass, $user_uid, $user_gid ) = getpwnam($user_name)
-      or croak 'user not in password database';
-    $uid = $user_uid;
+
+  # Only customize user if have something from caller
+  if ( defined $user_name and $user_name ne '' ) {
+    if ( $user_name =~ m/^([0-9]+)$/ ) {
+      $uid = $1;
+    } else {
+      ( $login, $pass, $user_uid, $user_gid ) = getpwnam($user_name)
+        or croak 'user not in password database';
+      $uid = $user_uid;
+    }
   }
 
   # Only customize group if have something from caller
-  if ( defined $group_name ) {
+  if ( defined $group_name and $group_name ne '' ) {
     if ( $group_name =~ m/^([0-9]+)$/ ) {
-      $gid = $group_name;
+      $gid = $1;
     } else {
       my ( $group_name, $pass, $group_gid ) = getgrnam($group_name)
         or croak 'group not in group database';
@@ -633,14 +664,14 @@ not, the module throws an error.
 =item B<mode> => I<unix mode>
 
 Accepts a Unix mode for C<chmod> to be applied to the file. Usual
-throwing of error. NOTE: depending on the source of the mode, C<oct>
-may be required to convert it:
+throwing of error. If the mode is a string starting with C<0>,
+C<oct> is used to convert it:
 
   my $orig_mode = (stat $source_file)[2] & 07777;
   ...->write_file({ ..., mode => $orig_mode });
 
   my $mode = '0644';
-  ...->write_file({ ..., mode => oct($mode) });
+  ...->write_file({ ..., mode => $mode });
 
 The module does not change C<umask>, nor is there a means to specify
 the permissions on directories created if B<MKPATH> is set.
@@ -674,6 +705,12 @@ written. To avoid this (slight but non-zero) risk, use the B<tmpdir>
 option to write the configuration out in full under a different
 directory on the same partition.
 
+=item B<backup> => I<suffix>
+
+Make a backup with this (non-empty) suffix. The backup is always created,
+even if there was no change. If a previous backup existed, it is deleted
+first. Usual throwing of error.
+
 =item B<checksum> => I<sha1 hexdigest>
 
 If this option exists, and B<CHECKSUM> is true, the module will not
@@ -780,7 +817,7 @@ L<http://danluu.com/file-consistency/>
 
 thrig - Jeremy Mates (cpan:JMATES) C<< <jmates at cpan.org> >>
 
-C<mtime> support contributed by Stijn De Weirdt.
+C<mtime> and other features contributed by Stijn De Weirdt.
 
 =head1 COPYRIGHT
 
